/**
 *	@file	ya_array_test.cpp
 *	@brief
 *	@author	tenk*
 *	@note
 *		array_test  arry_test2 ͍̈Ⴄ`FbN
 *		ŁAӖdĂe
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <algorithm>
#include <string>
#include "../ya_array.h"
#include "unitst.h"


UNITST_TEST(array_test) {
	ya_array<char, 10>	ac10;
	char	buf[256];
	memset(buf, 0, sizeof buf);
	UNITST_EQ_I(ac10.size(), 10);
	UNITST_EQ_I(ac10.capacity(), 10);
	UNITST_EQ_I(ac10.max_size(), 10);
	UNITST_EQ_I(ac10.empty(), 0);
	ac10.assign(10, '\0');
	UNITST_EQ_MEM(&ac10[0], buf, 10);
	UNITST_CHECK(ac10 == ac10);

	for (unsigned i = 0; i < 9; ++i) {
		ac10[i] = '0' + i;
		UNITST_EQ_I(ac10[i], char('0'+i));
	}

	ya_array<char, 10> ac10b = ac10;
	UNITST_EQ_MEM(&ac10b[0], "012345678", 10);
	UNITST_CHECK(ac10 == ac10b);
	UNITST_CHECK(ac10 >= ac10b);
	UNITST_CHECK(ac10 <= ac10b);
	ac10b[0] = '9';
	UNITST_CHECK(ac10 != ac10b);
	UNITST_CHECK(ac10 <= ac10b);
	UNITST_CHECK(ac10 <  ac10b);
	UNITST_CHECK(ac10b != ac10);
	UNITST_CHECK(ac10b >= ac10);
	UNITST_CHECK(ac10b >  ac10);
	ac10 = ac10b;
	UNITST_CHECK(ac10 == ac10b);
	UNITST_CHECK(ac10 >= ac10b);
	UNITST_CHECK(ac10 <= ac10b);

	UNITST_EQ_I(ac10.front(), '9');
	UNITST_EQ_I(ac10.back(), '\0');
	UNITST_EQ_I(*ac10.begin(), '9');
	UNITST_EQ_I(*(ac10.end() - 1), '\0');
	UNITST_EQ_I(*(ac10.end() - 2), '8');
	UNITST_EQ_I(*ac10.rbegin(), '\0');
	UNITST_EQ_I(*(ac10.rend()-1), '9');

	for (unsigned i = 0; i < 9; ++i) {
		ac10.at(i) = 'A' + i;
		UNITST_EQ_I(ac10.at(i), char('A'+i));
	}

	UNITST_ASSERT_THROW( ac10.at(11), std::out_of_range );
	//UNITST_ASSERT_ANY_THROW( ac10.at(11) );
	UNITST_ASSERT_ANY_THROW( ac10.at(unsigned(-1)) );


	ac10b.assign(10, '@');
	std::swap(ac10, ac10b);
	UNITST_EQ_MEM(&ac10 [0], "@@@@@@@@@@", 10);
	UNITST_EQ_MEM(&ac10b[0], "ABCDEFGHI\0", 10);
}



UNITST_TEST(array_test2) {
	ya_array<int, 5> ary;

	UNITST_EQ_I(ary.size(), 5);
	UNITST_EQ_I(ary.max_size(), 5);
	UNITST_EQ_I(ary.capacity(), 5);
	UNITST_EQ_I(ary.empty()   , 0);

	UNITST_ASSERT_NO_THROW(ary.at(4)=4);
	UNITST_ASSERT_THROW(ary.at(5)=5, std::out_of_range );

	int n = 0;
	for (n = 0; n < 5; ++n)
		ary[n] = n;
	for (n = 0; n < 5; ++n)
		ary[n] = ary[n]+1;

	UNITST_EQ_I(ary.front(), 1);
	UNITST_EQ_I(ary.back(), 5);

	ya_array<int, 5>::iterator it;
	for (n = 1, it = ary.begin(); it != ary.end(); ++it, ++n)
		UNITST_EQ(*it, n);

	ya_array<int, 5>::reverse_iterator ri;
	for (n = 5, ri = ary.rbegin(); ri != ary.rend(); ++ri, --n)
		UNITST_EQ(*ri, n);

	ya_array<int, 5> ary2(ary);
	UNITST_EQ_MEM(&ary2[0], &ary[0], sizeof ary);
	UNITST_EQ_I(ary == ary2, true );
	UNITST_EQ_I(ary != ary2, false);
	UNITST_EQ_I(ary <  ary2, false);
	UNITST_EQ_I(ary <= ary2, true );
	UNITST_EQ_I(ary >  ary2, false);
	UNITST_EQ_I(ary >= ary2, true );

	ya_array<int, 5> ary3 = ary;
	UNITST_EQ_MEM(&ary3[0], &ary[0], sizeof ary);
	UNITST_EQ(ary3, ary);
	ary3[2] = 10;
	UNITST_EQ_I((ary == ary3), false);
	UNITST_EQ_I((ary != ary3), true );
	UNITST_EQ_I((ary <	ary3), true );
	UNITST_EQ_I((ary <= ary3), true );
	UNITST_EQ_I((ary >	ary3), false);
	UNITST_EQ_I((ary >= ary3), false);
	ary3[2] = 1;
	UNITST_EQ_I((ary == ary3), false);
	UNITST_EQ_I((ary != ary3), true );
	UNITST_EQ_I((ary <	ary3), false);
	UNITST_EQ_I((ary <= ary3), false);
	UNITST_EQ_I((ary >	ary3), true );
	UNITST_EQ_I((ary >= ary3), true );

	ya_array<int, 5> ary4 = ary3;

	ary2.swap(ary4);
	UNITST_EQ(ary2, ary3);
	UNITST_EQ(ary4, ary);

	std::swap(ary2, ary4);
	UNITST_EQ(ary2, ary);
	UNITST_EQ(ary4, ary3);

	//ya_array<int, 10> ary5 = ary;
	ya_array<int, 10> ary5;
	ary5.assign(ary.begin(), ary.end());

	UNITST_EQ_MEM(&ary5[0], &ary[0], sizeof ary);

	for (n = 0; n < int(ary5.size()); ++n)
		ary5[n] = (n+1)*2;
	//x ary4 = ary5;
	//ary4.assign(ary5.begin(), ary5.begin()+5);
	//UNITST_EQ_MEM(&ary4[0], &ary5[0], sizeof ary4);

	ary4.assign(5U, 500);
	for (n = 0; n < 5; ++n)
		UNITST_EQ_I(ary4[n], 500);
}



UNITST_TEST(array_test3) {
  #ifdef __BORLANDC__
	#define S(s)		std::string(s)
  #else
	#define S(s)		s
  #endif
	ya_array<std::string, 5> ary;
	UNITST_EQ_I(ary.size(), 5);
	UNITST_EQ_I(ary.max_size(), 5);
	UNITST_EQ_I(ary.capacity(), 5);
	UNITST_EQ_I(ary.empty()   , 0);

	UNITST_ASSERT_NO_THROW(ary.at(4)=4);
	UNITST_ASSERT_THROW(ary.at(5)=5, std::out_of_range );

	static const char* strs[] = {"0","1","2","3","4","5","6"};
	int n = 0;

	for (n = 0; n < 5; ++n)
		ary[n] = strs[n];

	ya_array<std::string, 5>::iterator it;
	for (n = 0, it = ary.begin(); it != ary.end(); ++it, ++n)
		UNITST_EQ_STRING(*it, S(strs[n]));

	ya_array<std::string, 5>::reverse_iterator ri;
	for (n = 4, ri = ary.rbegin(); ri != ary.rend(); ++ri, --n)
		UNITST_EQ_STRING(*ri, S(strs[n]));

	for (n = 0; n < 5; ++n)
		ary[n] = ary[n] + "_str";

	UNITST_EQ_STRING(ary.front(), S("0_str"));
	UNITST_EQ_STRING(ary.back(),  S("4_str"));

	ya_array<std::string, 5> ary2(ary);
	for (unsigned i = 0; i < 5; ++i)
		UNITST_EQ_STRING(ary2[i], ary[i]);
	UNITST_EQ_I(ary == ary2, true );
	UNITST_EQ_I(ary != ary2, false);
	UNITST_EQ_I(ary <  ary2, false);
	UNITST_EQ_I(ary <= ary2, true );
	UNITST_EQ_I(ary >  ary2, false);
	UNITST_EQ_I(ary >= ary2, true );

	ya_array<std::string, 5> ary3 = ary;
	for (unsigned i = 0; i < 5; ++i)
		UNITST_EQ_STRING(ary3[i], ary[i]);
	ary3[2] = "9";
	UNITST_EQ_I((ary == ary3), false);
	UNITST_EQ_I((ary != ary3), true );
	UNITST_EQ_I((ary <	ary3), true );
	UNITST_EQ_I((ary <= ary3), true );
	UNITST_EQ_I((ary >	ary3), false);
	UNITST_EQ_I((ary >= ary3), false);
	ary3[2] = 1;
	UNITST_EQ_I((ary == ary3), false);
	UNITST_EQ_I((ary != ary3), true );
	UNITST_EQ_I((ary <	ary3), false);
	UNITST_EQ_I((ary <= ary3), false);
	UNITST_EQ_I((ary >	ary3), true );
	UNITST_EQ_I((ary >= ary3), true );

	ya_array<std::string, 5> ary4 = ary3;

	ary2.swap(ary4);
	UNITST_EQ(ary2, ary3);
	UNITST_EQ(ary4, ary);

	std::swap(ary2, ary4);
	UNITST_EQ(ary2, ary);
	UNITST_EQ(ary4, ary3);

	//ya_array<int, 10> ary5 = ary;
	ya_array<std::string, 10> ary5;
	ary5.assign(ary.begin(), ary.end());
	for (n = 0; n < 5; ++n)
		UNITST_EQ_STRING(ary5[n], ary[n]);

	ary4.assign(5U, std::string("str"));
	for (n = 0; n < 5; ++n)
		UNITST_EQ_STRING(ary4[n], S("str"));
	#undef S
}
