/**
 *	@file lexicographical_compare.hpp
 *	@brief std::lexicographical_compare ̑pi.
 *	@author	tenk*
 *	@note
 *	-	open-watcom 1.8ɂȂ̂ō쐬.
 *	-	Public Domain Software
 */

#ifndef LEXICOGRAPHICAL_COMPARE_HPP
#define LEXICOGRAPHICAL_COMPARE_HPP

namespace std {

template<class Ite1, class Ite2> inline
bool lexicographical_compare(Ite1 first1, Ite1 last1, Ite2 first2, Ite2 last2) {
	Ite1 i1 = first1;
	Ite2 i2 = first2;
	while (i1 != last1) {
		if (i2 == last2) return 0;
		if (*i1 < *i2)	 return 1;
		if (*i2 < *i1)	 return 0;
		++i1;
		++i2;
	}
	return (i2 != last2);
}


template<class Ite1, class Ite2, class Pred> inline
bool lexicographical_compare(Ite1 first1, Ite1 last1, Ite2 first2, Ite2 last2, Pred pred) {
	Ite1 i1 = first1;
	Ite2 i2 = first2;
	while (i1 != last1) {
		if (i2 == last2)	return 0;
		if (pred(*i1, *i2)) return 1;
		if (pred(*i2, *i1)) return 0;
		++i1;
		++i2;
	}
	return (i2 != last2);
}


}	// namespace std
#endif	// LEXICOGRAPHICAL_COMPARE_HPP
